#  ICE Revision: $Id: FileBasis.py 7793 2007-08-19 18:55:31Z bgschaid $ 
"""Basis for the handling of OpenFOAM-files

Transparently accepts gnuzipped files"""

import os,re
from os import path
from tempfile import mktemp
import gzip


from PyFoam.Basics.Utilities import Utilities
from PyFoam.Basics.LineReader import LineReader

class FileBasis(Utilities):
    """ Base class for the other OpenFOAM--file-classes"""
    
    removedString="//PyFoamRemoved: "
    """Comment for lines that were overwritten by PyFoam-routines"""
    
    addedString="//PyFoamAdded"
    """Comment for lines that were added by PyFoam-routines"""
    
    def __init__(self,name):
        """@param name: Name of the file. If the field is zipped the .gz is
        appended"""
        self.name = path.abspath(name)
        if path.exists(self.name):
            self.zipped=False
            if path.splitext(self.name)[1]==".gz":
                self.zipped=True
        else:
            self.zipped=True
            self.name+=".gz"

        self.fh=None
        self.content=None

    def openFile(self,keepContent=False,mode="r"):
        """opens the file. To be overloaded by derived classes"""
        if not keepContent:
            self.content=None
        if self.zipped:
            self.fh=gzip.open(self.name,mode)
        else:
            self.fh=open(self.name,mode)

    def closeFile(self):
        """ closes the file"""
        self.fh.close()
        self.fh=None

    def readFile(self):
        """ read the whole File into memory"""
        self.openFile()
        self.content=self.parse(self.fh.read())
        self.closeFile()

    def writeFile(self):
        """ write the whole File from memory"""
        if self.content!=None:
            self.openFile(keepContent=True,mode="w")
            self.fh.write(str(self))
            self.closeFile()

    def parse(self,cnt):
        """ Parse a string that is to be the content, to be overriden
        by the sub-classes"""

        return cnt

    def __str__(self):
        """Build a string from self.content, to be overriden by sub-classes"""

        return self.content
    
    def makeTemp(self):
        """creates a temporary file"""
        fn=mktemp(dir=path.dirname(self.name))
        if self.zipped:
            fh=gzip.open(fn,"w")
        else:
            fh=open(fn,"w")

        return fh,fn

    def goTo(self,l,s,out=None,echoLast=False,stop=None):
        """Read lines until a token is found

        @param l: a LineReader object
        @param s: the string to look for
        @param out: filehandle to echo the lines to
        @param stop: pattern that indicates that exp will never be found (only passed through to goMatch)
        @param echoLast: echo the line with the string"""
        exp=re.compile("( |^)"+s+"( |$)")
        m=self.goMatch(l,exp,out=out,stop=stop)
        if out!=None and echoLast:
            out.write(l.line+"\n")
            
    def goMatch(self,l,exp,out=None,stop=None):
        """Read lines until a regular expression is matched

        @param l: a LineReader object
        @param exp: the expression to look for
        @param out: filehandle to echo the lines to
        @param stop: pattern that indicates that exp will never be found
        @return: match-object if exp is found, the line if stop is found and None if the end of the file is reached"""        
        while l.read(self.fh):
            m=exp.match(l.line)
            if m!=None:
                return m
            elif stop!=None:
                if stop.match(l.line):
                    return l.line
            if out!=None:
                out.write(l.line+"\n")

        return None
    
    def copyRest(self,l,out):
        """Copy the rest of the file
        
        @param l: a LineReader object
        @param out: filehandle to echo the lines to"""        
        while l.read(self.fh):
            out.write(l.line+"\n")

    def purgeFile(self):
        """Undo all the manipulations done by PyFOAM

        Goes through the file and removes all lines that were added"""
        rmExp= re.compile("^"+self.removedString+"(.*)$")
        addExp=re.compile("^(.*)"+self.addedString+"$")
        
        l=LineReader()
        self.openFile()

        (fh,fn)=self.makeTemp()

        while l.read(self.fh):
            toPrint=l.line

            m=addExp.match(l.line)
            if m!=None:
                continue
            
            m=rmExp.match(l.line)
            if m!=None:
                toPrint=m.group(1)
                
            fh.write(toPrint+"\n")
        
        self.closeFile()
        fh.close()
        os.rename(fn,self.name)

class FileBasisBackup(FileBasis):
    """A file with a backup-copy"""

    def __init__(self,name,backup=False):
        """@param name: The name of the parameter file
        @type name: str
        @param backup: create a backup-copy of the file
        @type backup: boolean"""
        
        FileBasis.__init__(self,name)

        if backup:
            self.backupName=self.name+".backup"
            self.execute("cp "+self.name+" "+self.backupName)
        else:
            self.backupName=None

    def restore(self):
        """if a backup-copy was made the file is restored from this"""
        if self.backupName!=None:
            self.execute("cp "+self.backupName+" "+self.name)
    
